/**
 * 
 */
package ima.uco.fr.nuit.model;

import ima.uco.fr.nuit.checker.CheckSolutionHandler;
import ima.uco.fr.nuit.output.LaTeXOutputter;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Scanner;

/**
 * Classe principale
 * 
 * @author IMAvengers
 * 
 */
public class Main {

	private static int nbLigne;			// Dimension du panneau publicitaire
	private static int[] pourcentage;	// Pourcentages de nombre de femmes par diagonales pouvant tre obtenus

	/**
	 * Mthode principale
	 * @param args : vide
	 */
	@SuppressWarnings("resource")
	public static void main(String[] args) {
		// Initialise de l'encart publicitaire selon l'utilisateur 
		Scanner clavier = new Scanner(System.in); 
		System.out.println("Veuillez rentrer le nombre de ligne de l'encart publicitaire :");
		nbLigne = clavier.nextInt();

		// Calcul des diffrents pourcentages de nombre de femmes par diagonales pouvant tre obtenus
		percentageCalculation();

		// Construit le modle mathmatique utilisant CPLEX
		Model model = new Model(nbLigne,nbLigne,pourcentage);

		// Rsout le modle
		if (model.resolve()) {
			// Obtention de la matrice reprsentant la position des femmes dans l'encart
			int[][] x = model.obtainMatrix();

			// Vrifie si la solution est correcte en utilisant le vrificateur de solution
			CheckSolutionHandler.checkSolution(x);

			// Export des solutions sous fichier CSV
			model.exportToCSV("solutionCSV/");

			// Export des solutions au format LaTeX
			try {
				String file = "solutionLaTeX/solution_" + nbLigne + ".tex";
				PrintWriter out = new PrintWriter(new FileWriter(file));

				out.println(LaTeXOutputter.getMatrix(x) + "\n");
				out.println(LaTeXOutputter.getFigure(x));

				out.close();
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Calcul des diffrents pourcentages de nombre de femmes par diagonales pouvant tre obtenus
	 */
	private static void percentageCalculation() {
		// Initialise les champs
		ArrayList<Integer> nombres = new ArrayList<Integer>();	
		int temp;	

		// Calcul du multiplicateur 
		int multiplicateur = calculeMultiplicateur()*100;

		// Calcul des diffrents pourcentages possibles
		nombres.add(0);
		nombres.add(multiplicateur);

		for (int i=1 ; i<=nbLigne ; i++) {
			for (int j=1 ; j<i ; j++) {
				temp = j*multiplicateur/i;
				if(!nombres.contains(temp))
					nombres.add(temp);
			}
		}

		// Tri de la liste des pourcentages dans l'ordre croissant
		Collections.sort(nombres);

		// Mise en forme 'tableau' de notre liste
		pourcentage = new int[nombres.size()];
		for(int i=0 ; i<nombres.size() ; i++)
			pourcentage[i] = nombres.get(i);
	}

	/**
	 * Calcul du multiplicateur, qui est le PPCM des nombres compris entre 1 et le nombre de lignes
	 * @return multiplicateur
	 */
	public static int calculeMultiplicateur() {
		// Initialise les champs
		int x=1, y=0;

		for (int i = 2; i < nbLigne+1; i++){
			y = calculePPCM(x, i);
			x = y;
		}

		return y;
	}

	/**
	 * Calcul du PPCM de nb1 et nb2
	 * @param nb1 : premier nombre
	 * @param nb2 : second nombre
	 * @return le PPCM des deux nombres
	 */
	private static int calculePPCM(int nb1, int nb2) {
		// Initialise les champs
		int produit = nb1*nb2;
		int reste   = nb1%nb2;

		while(reste != 0){
			nb1 = nb2;
			nb2 = reste;
			reste = nb1%nb2;
		}

		return produit/nb2;		
	}

}
